@@ -22,7 +22,7 @@
  */
 
 #include "backgroundparser.h"
- 
+
 #include <QList>
 #include <QFile>
 #include <QTimer>
@@ -30,6 +30,7 @@
 #include <QWaitCondition>
 #include <QMutexLocker>
 #include <QThread>
+#include <QtCore/QWeakPointer>
 
 #include <kdebug.h>
 #include <kglobal.h>
@@ -37,8 +38,6 @@
 #include <ksharedconfig.h>
 #include <klocale.h>
 
-#include <ktexteditor/smartrange.h>
-#include <ktexteditor/smartinterface.h>
 #include <ktexteditor/document.h>
 
 #include <threadweaver/State.h>
@@ -54,8 +53,11 @@
 #include "parsejob.h"
 #include "parserdependencypolicy.h"
 #include <editor/modificationrevisionset.h>
+#include <interfaces/icore.h>
+#include <qcoreapplication.h>
+#include <interfaces/idocumentcontroller.h>
+#include <interfaces/isession.h>
 
-//@todo Enable this again once languageForUrl is thread-safe
 const bool separateThreadForHighPriority = true;
 
 namespace KDevelop
@@ -65,10 +67,10 @@ class BackgroundParserPrivate
 {
 public:
     BackgroundParserPrivate(BackgroundParser *parser, ILanguageController *languageController)
-        :m_parser(parser), m_languageController(languageController)
+        :m_parser(parser), m_languageController(languageController), m_shuttingDown(false), m_mutex(QMutex::Recursive)
     {
         parser->d = this; //Set this so we can safely call back BackgroundParser from within loadSettings()
-        
+
         m_timer.setSingleShot(true);
         m_delay = 500;
         m_threads = 1;
@@ -79,14 +81,12 @@ public:
         ThreadWeaver::setDebugLevel(true, 1);
 
         QObject::connect(&m_timer, SIGNAL(timeout()), m_parser, SLOT(parseDocuments()));
-
-        loadSettings(); // Start the weaver
     }
-    
+
     void startTimerThreadSafe() {
         QMetaObject::invokeMethod(m_parser, "startTimer", Qt::QueuedConnection);
     }
-    
+
     ~BackgroundParserPrivate()
     {
         suspend();
@@ -102,72 +102,67 @@ public:
             it.value()->setBackgroundParser(0);
             delete it.value();
         }
-
-        qDeleteAll(m_delayedParseJobs);
     }
 
     // Non-mutex guarded functions, only call with m_mutex acquired.
     void parseDocumentsInternal()
     {
+        if(m_shuttingDown)
+            return;
         // Create delayed jobs, that is, jobs for documents which have been changed
         // by the user.
         QList<ParseJob*> jobs;
-        
-        for(QHash<KUrl, DocumentChangeTracker*>::iterator it = m_delayedParseJobs.begin(); it != m_delayedParseJobs.end(); ) {
-            KUrl url(it.key());
-            
-            if(m_parseJobs.contains(url)) {
-                kDebug() << "already parsing" << url << ", delaying the parse-job";
-                ++it; //Add the delayed job later
-            }else{
-                ParseJob* job = createParseJob(url, 
-                                            TopDUContext::AllDeclarationsContextsAndUses,
-                                            QList<QPointer<QObject> >());
-                if (job) {
-                    job->setChangedRanges(it.value()->changedRanges());
-                    jobs.append(job);
-                    specialParseJob = job;
-                } else {
-                    kWarning() << "No job created for url " << it.key();
-                }
-                delete *it;
-                it = m_delayedParseJobs.erase(it);
-            }
-        }
 
         for (QMap<int, QSet<KUrl> >::Iterator it1 = m_documentsForPriority.begin();
              it1 != m_documentsForPriority.end(); ++it1 )
         {
             if(it1.key() > m_neededPriority)
                 break; //The priority is not good enough to be processed right now
-            
+
             for(QSet<KUrl>::Iterator it = it1.value().begin(); it != it1.value().end();) {
                 //Only create parse-jobs for up to thread-count * 2 documents, so we don't fill the memory unnecessarily
                 if(m_parseJobs.count() >= m_threads+1 || (m_parseJobs.count() >= m_threads && !separateThreadForHighPriority) )
                     break;
-                
+
                 if(m_parseJobs.count() >= m_threads && it1.key() > BackgroundParser::NormalPriority && !specialParseJob)
                     break; //The additional parsing thread is reserved for higher priority parsing
-                
+
                 // When a document is scheduled for parsing while it is being parsed, it will be parsed
                 // again once the job finished, but not now.
                 if (m_parseJobs.contains(*it) ) {
                     ++it;
                     continue;
                 }
-                
+
                 kDebug(9505) << "creating parse-job" << *it << "new count of active parse-jobs:" << m_parseJobs.count() + 1;
-                ParseJob* job = createParseJob(*it, m_documents[*it].features(), m_documents[*it].notifyWhenReady());
-                
+                Q_ASSERT(m_documents.contains(*it));
+                const DocumentParsePlan& parsePlan = m_documents[*it];
+                ParseJob* job = createParseJob(*it, parsePlan.features(), parsePlan.notifyWhenReady());
+
                 if(m_parseJobs.count() == m_threads+1 && !specialParseJob)
                     specialParseJob = job; //This parse-job is allocated into the reserved thread
-                
+
                 if(job)
                     jobs.append(job);
-                
+
+                // Remove all mentions of this document.
+                foreach(DocumentParseTarget target, parsePlan.targets) {
+                    if (target.priority != it1.key()) {
+                        m_documentsForPriority[target.priority].remove(*it);
+                    }
+                }
                 m_documents.remove(*it);
                 it = it1.value().erase(it);
                 --m_maxParseJobs; //We have added one when putting the document into m_documents
+                
+                if(!m_documents.isEmpty())
+                {
+                    // Only try creating one parse-job at a time, else we might iterate through thousands of files
+                    // without finding a language-support, and block the UI for a long time.
+                    // If there are more documents to parse, instantly re-try.
+                    QMetaObject::invokeMethod(m_parser, "parseDocuments", Qt::QueuedConnection);
+                    break;
+                }
             }
         }
 
@@ -179,14 +174,25 @@ public:
         m_parser->updateProgressBar();
 
         //We don't hide the progress-bar in updateProgressBar, so it doesn't permanently flash when a document is reparsed again and again.
-        if(m_doneParseJobs == m_maxParseJobs)
+        if(m_doneParseJobs == m_maxParseJobs
+            || (m_neededPriority == BackgroundParser::BestPriority && m_weaver.queueLength() == 0))
+        {
             emit m_parser->hideProgress(m_parser);
+        }
     }
 
-    ParseJob* createParseJob(const KUrl& url, TopDUContext::Features features, QList<QPointer<QObject> > notifyWhenReady)
+    ParseJob* createParseJob(const KUrl& url, TopDUContext::Features features, QList<QWeakPointer<QObject> > notifyWhenReady)
     {
         QList<ILanguage*> languages = m_languageController->languagesForUrl(url);
         foreach (ILanguage* language, languages) {
+            if(!language) {
+                kWarning() << "got zero language for" << url;
+                continue;
+            }
+            if(!language->languageSupport()) {
+                kWarning() << "language has no language support assigned:" << language->name();
+                continue;
+            }
             ParseJob* job = language->languageSupport()->createParseJob(url);
             if (!job) {
                 continue; // Language part did not produce a valid ParseJob.
@@ -195,6 +201,7 @@ public:
             job->setMinimumFeatures(features);
             job->setBackgroundParser(m_parser);
             job->setNotifyWhenReady(notifyWhenReady);
+            job->setTracker(m_parser->trackerForUrl(IndexedString(url)));
 
             QObject::connect(job, SIGNAL(done(ThreadWeaver::Job*)),
                                 m_parser, SLOT(parseComplete(ThreadWeaver::Job*)));
@@ -212,7 +219,15 @@ public:
         }
 
         if(languages.isEmpty())
-            kWarning(9504) << "found no languages for url" << url;
+            kDebug() << "found no languages for url" << url;
+        else
+            kDebug() << "could not create parse-job for url" << url;
+
+        //Notify that we failed
+        typedef QWeakPointer<QObject> Notify;
+        foreach(const Notify& n, notifyWhenReady)
+            if(n)
+                QMetaObject::invokeMethod(n.data(), "updateReady", Qt::QueuedConnection, Q_ARG(KDevelop::IndexedString, IndexedString(url)), Q_ARG(KDevelop::ReferencedTopDUContext, ReferencedTopDUContext()));
 
         return 0;
     }
@@ -221,17 +236,25 @@ public:
     void loadSettings()
     {
         ///@todo re-load settings when they have been changed!
-        KConfigGroup config(KGlobal::config(), "Background Parser");
+        Q_ASSERT(ICore::self()->activeSession());
+        KConfigGroup config(ICore::self()->activeSession()->config(), "Background Parser");
 
-        m_delay = config.readEntry("Delay", 500);
+        // stay backwards compatible
+        KConfigGroup oldConfig(KGlobal::config(), "Background Parser");
+#define BACKWARDS_COMPATIBLE_ENTRY(entry, default) \
+config.readEntry(entry, oldConfig.readEntry(entry, default))
+
+        m_delay = BACKWARDS_COMPATIBLE_ENTRY("Delay", 500);
         m_timer.setInterval(m_delay);
         m_threads = 0;
-        m_parser->setThreadCount(config.readEntry("Number of Threads", 1));
+        m_parser->setThreadCount(BACKWARDS_COMPATIBLE_ENTRY("Number of Threads", 1));
 
-        if (config.readEntry("Enabled", true)) {
-            resume();
+        resume();
+
+        if (BACKWARDS_COMPATIBLE_ENTRY("Enabled", true)) {
+            m_parser->enableProcessing();
         } else {
-            suspend();
+            m_parser->disableProcessing();
         }
     }
 
@@ -265,20 +288,26 @@ public:
     ILanguageController* m_languageController;
 
     //Current parse-job that is executed in the additional thread
-    QPointer<ParseJob> specialParseJob;
-    
+    QWeakPointer<ParseJob> specialParseJob;
+
     QTimer m_timer;
     int m_delay;
     int m_threads;
+    
+    bool m_shuttingDown;
+    
     struct DocumentParseTarget {
-        QPointer<QObject> notifyWhenReady;
+        QWeakPointer<QObject> notifyWhenReady;
         int priority;
         TopDUContext::Features features;
+        bool operator==(const DocumentParseTarget& rhs) const {
+            return notifyWhenReady == rhs.notifyWhenReady && priority == rhs.priority && features == rhs.features;
+        }
     };
 
     struct DocumentParsePlan {
         QList<DocumentParseTarget> targets;
-        
+
         int priority() const {
             //Pick the best priority
             int ret = BackgroundParser::WorstPriority;
@@ -287,7 +316,7 @@ public:
                     ret = target.priority;
             return ret;
         }
-        
+
         TopDUContext::Features features() const {
             //Pick the best features
             TopDUContext::Features ret = (TopDUContext::Features)0;
@@ -295,25 +324,27 @@ public:
                 ret = (TopDUContext::Features) (ret | target.features);
             return ret;
         }
-        
-        QList<QPointer<QObject> > notifyWhenReady() {
-            QList<QPointer<QObject> > ret;
-            
+
+        QList<QWeakPointer<QObject> > notifyWhenReady() const {
+            QList<QWeakPointer<QObject> > ret;
+
             foreach(const DocumentParseTarget &target, targets)
                 if(target.notifyWhenReady)
                     ret << target.notifyWhenReady;
-            
+
             return ret;
         }
     };
-    // A list of known documents, and their priority
+    // A list of documents that are planned to be parsed, and their priority
     QMap<KUrl, DocumentParsePlan > m_documents;
+    // The documents ordered by priority
     QMap<int, QSet<KUrl> > m_documentsForPriority;
-    // Current parse jobs
+    // Currently running parse jobs
     QHash<KUrl, ParseJob*> m_parseJobs;
-    QHash<KUrl, DocumentChangeTracker*> m_delayedParseJobs;
-
-    QHash<KTextEditor::SmartRange*, KUrl> m_managedRanges;
+    // A change tracker for each managed document
+    QHash<IndexedString, DocumentChangeTracker*> m_managed;
+    // The url for each managed document. Those may temporarily differ from the real url.
+    QHash<KTextEditor::Document*, IndexedString> m_managedTextDocumentUrls;
 
     ThreadWeaver::Weaver m_weaver;
     ParserDependencyPolicy m_dependencyPolicy;
@@ -326,10 +357,19 @@ public:
     int m_neededPriority; //The minimum priority needed for processed jobs
 };
 
-
 BackgroundParser::BackgroundParser(ILanguageController *languageController)
     : QObject(languageController), d(new BackgroundParserPrivate(this, languageController))
 {
+    Q_ASSERT(ICore::self()->documentController());
+    connect(ICore::self()->documentController(), SIGNAL(documentLoaded(KDevelop::IDocument*)), this, SLOT(documentLoaded(KDevelop::IDocument*)));
+    connect(ICore::self()->documentController(), SIGNAL(documentUrlChanged(KDevelop::IDocument*)), this, SLOT(documentUrlChanged(KDevelop::IDocument*)));
+    connect(ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)), this, SLOT(documentClosed(KDevelop::IDocument*)));
+    connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), this, SLOT(aboutToQuit()));
+}
+
+void BackgroundParser::aboutToQuit()
+{
+    d->m_shuttingDown = true;
 }
 
 BackgroundParser::~BackgroundParser()
@@ -355,18 +395,11 @@ void BackgroundParser::clear(QObject* parent)
     }
 }
 
-void BackgroundParser::loadSettings(bool projectIsLoaded)
+void BackgroundParser::loadSettings()
 {
-    Q_UNUSED(projectIsLoaded)
-
     d->loadSettings();
 }
 
-void BackgroundParser::saveSettings(bool projectIsLoaded)
-{
-    Q_UNUSED(projectIsLoaded)
-}
-
 void BackgroundParser::parseProgress(KDevelop::ParseJob* job, float value, QString text)
 {
     Q_UNUSED(text)
@@ -374,37 +407,30 @@ void BackgroundParser::parseProgress(KDevelop::ParseJob* job, float value, QStri
     updateProgressBar();
 }
 
-// void BackgroundParser::addUpdateJob(ReferencedTopDUContext topContext, TopDUContext::Features features, QObject* notifyWhenReady, int priority)
-// {
-//     QMutexLocker lock(&d->m_mutex);
-//     {
-//     }
-// }
-
 void BackgroundParser::revertAllRequests(QObject* notifyWhenReady)
 {
-    QPointer<QObject> p(notifyWhenReady);
-    
+    QWeakPointer<QObject> p(notifyWhenReady);
+
     QMutexLocker lock(&d->m_mutex);
     for(QMap<KUrl, BackgroundParserPrivate::DocumentParsePlan >::iterator it = d->m_documents.begin(); it != d->m_documents.end(); ) {
-        
+
         d->m_documentsForPriority[it.value().priority()].remove(it.key());
-        
+
         int index = -1;
         for(int a = 0; a < (*it).targets.size(); ++a)
-            if((*it).targets[a].notifyWhenReady == notifyWhenReady)
+            if((*it).targets[a].notifyWhenReady.data() == notifyWhenReady)
                 index = a;
-        
+
         if(index != -1) {
             (*it).targets.removeAt(index);
             if((*it).targets.isEmpty()) {
                 it = d->m_documents.erase(it);
                 --d->m_maxParseJobs;
-                
+
                 continue;
             }
         }
-        
+
         d->m_documentsForPriority[it.value().priority()].insert(it.key());
         ++it;
     }
@@ -413,32 +439,30 @@ void BackgroundParser::revertAllRequests(QObject* notifyWhenReady)
 void BackgroundParser::addDocument(const KUrl& url, TopDUContext::Features features, int priority, QObject* notifyWhenReady)
 {
 //     kDebug(9505) << "BackgroundParser::addDocument" << url.prettyUrl();
+    KUrl cleanedUrl = url;
+    cleanedUrl.cleanPath();
     QMutexLocker lock(&d->m_mutex);
     {
-        Q_ASSERT(url.isValid());
-        
+        Q_ASSERT(cleanedUrl.isValid());
+
         BackgroundParserPrivate::DocumentParseTarget target;
         target.priority = priority;
         target.features = features;
-        target.notifyWhenReady = QPointer<QObject>(notifyWhenReady);
-        
-        QMap<KUrl, BackgroundParserPrivate::DocumentParsePlan >::iterator it = d->m_documents.find(url);
-        
+        target.notifyWhenReady = QWeakPointer<QObject>(notifyWhenReady);
+
+        QMap<KUrl, BackgroundParserPrivate::DocumentParsePlan>::iterator it = d->m_documents.find(cleanedUrl);
+
         if (it != d->m_documents.end()) {
             //Update the stored plan
-            
-            d->m_documentsForPriority[it.value().priority()].remove(url);
+
+            d->m_documentsForPriority[it.value().priority()].remove(cleanedUrl);
             it.value().targets << target;
-            d->m_documentsForPriority[it.value().priority()].insert(url);
-        }
-        
-        if (it == d->m_documents.end()) {
-//             kDebug(9505) << "BackgroundParser::addDocument: queuing" << url;
-            d->m_documents[url].targets << target;
-            d->m_documentsForPriority[d->m_documents[url].priority()].insert(url);
+            d->m_documentsForPriority[it.value().priority()].insert(cleanedUrl);
+        }else{
+//             kDebug(9505) << "BackgroundParser::addDocument: queuing" << cleanedUrl;
+            d->m_documents[cleanedUrl].targets << target;
+            d->m_documentsForPriority[d->m_documents[cleanedUrl].priority()].insert(cleanedUrl);
             ++d->m_maxParseJobs; //So the progress-bar waits for this document
-        } else {
-//             kDebug(9505) << "BackgroundParser::addDocument: is already queued:" << url;
         }
 
         d->startTimerThreadSafe();
@@ -451,16 +475,27 @@ void BackgroundParser::addDocumentList(const KUrl::List &urls, TopDUContext::Fea
         addDocument(url, features, priority);
 }
 
-void BackgroundParser::removeDocument(const KUrl &url)
+void BackgroundParser::removeDocument(const KUrl &url, QObject* notifyWhenReady)
 {
     QMutexLocker lock(&d->m_mutex);
 
     Q_ASSERT(url.isValid());
 
     if(d->m_documents.contains(url)) {
+        
         d->m_documentsForPriority[d->m_documents[url].priority()].remove(url);
-        d->m_documents.remove(url);
-        --d->m_maxParseJobs;
+        
+        foreach(BackgroundParserPrivate::DocumentParseTarget target, d->m_documents[url].targets)
+            if(target.notifyWhenReady.data() == notifyWhenReady)
+                d->m_documents[url].targets.removeAll(target);
+
+        if(d->m_documents[url].targets.isEmpty()) {
+            d->m_documents.remove(url);
+            --d->m_maxParseJobs;
+        }else{
+            //Insert with an eventually different priority
+            d->m_documentsForPriority[d->m_documents[url].priority()].insert(url);
+        }
     }
 }
 
@@ -473,23 +508,27 @@ void BackgroundParser::parseDocuments()
 
 void BackgroundParser::parseComplete(ThreadWeaver::Job* job)
 {
-    QMutexLocker lock(&d->m_mutex);
-
     if (ParseJob* parseJob = qobject_cast<ParseJob*>(job)) {
 
         emit parseJobFinished(parseJob);
 
-        d->m_parseJobs.remove(parseJob->document().str());
-
-        d->m_jobProgress.remove(parseJob);
-        
-        parseJob->setBackgroundParser(0);
-
-		delete parseJob;
-
-        ++d->m_doneParseJobs;
-        updateProgressBar();
-
+        {
+            {
+                QMutexLocker lock(&d->m_mutex);
+    
+                d->m_parseJobs.remove(parseJob->document().str());
+    
+                d->m_jobProgress.remove(parseJob);
+    
+                parseJob->setBackgroundParser(0);
+    
+                ++d->m_doneParseJobs;
+                updateProgressBar();
+            }
+            //Unlock the mutex before deleting the parse-job, because the parse-job
+            //has a virtual destructor that may lock the duchain, leading to deadlocks
+            delete parseJob;
+        }
         //Continue creating more parse-jobs
         QMetaObject::invokeMethod(this, "parseDocuments", Qt::QueuedConnection);
     }
@@ -505,6 +544,17 @@ void BackgroundParser::enableProcessing()
     setNeededPriority(WorstPriority);
 }
 
+bool BackgroundParser::isQueued(KUrl url) const {
+    QMutexLocker lock(&d->m_mutex);
+    return d->m_documents.contains(url);
+}
+
+int BackgroundParser::queuedCount() const
+{
+    QMutexLocker lock(&d->m_mutex);
+    return d->m_documents.count();
+}
+
 void BackgroundParser::setNeededPriority(int priority)
 {
     QMutexLocker lock(&d->m_mutex);
@@ -529,19 +579,20 @@ void BackgroundParser::resume()
 void BackgroundParser::updateProgressBar()
 {
     if (d->m_doneParseJobs >= d->m_maxParseJobs) {
-        if(d->m_doneParseJobs > d->m_maxParseJobs)
-            kWarning() << "m_doneParseJobs larger than m_maxParseJobs:" << d->m_doneParseJobs << d->m_maxParseJobs;
+        if(d->m_doneParseJobs > d->m_maxParseJobs) {
+            kDebug() << "m_doneParseJobs larger than m_maxParseJobs:" << d->m_doneParseJobs << d->m_maxParseJobs;
+        }
         d->m_doneParseJobs = 0;
         d->m_maxParseJobs = 0;
     } else {
         float additionalProgress = 0;
         for(QMap<KDevelop::ParseJob*, float>::const_iterator it = d->m_jobProgress.constBegin(); it != d->m_jobProgress.constEnd(); ++it)
             additionalProgress += *it;
-        
+
         emit showProgress(this, 0, d->m_maxParseJobs*1000, (additionalProgress + d->m_doneParseJobs)*1000);
     }
 }
- 
+
 ParserDependencyPolicy* BackgroundParser::dependencyPolicy() const
 {
     return &d->m_dependencyPolicy;
@@ -574,58 +625,89 @@ void BackgroundParser::setDelay(int miliseconds)
     }
 }
 
-void BackgroundParser::addManagedTopRange(const KUrl& document, KTextEditor::SmartRange* range)
-{
-    range->addWatcher(this);
-    d->m_managedRanges.insert(range, document);
-}
-
-void BackgroundParser::removeManagedTopRange(KTextEditor::SmartRange* range)
-{
-    range->removeWatcher(this);
-    d->m_managedRanges.remove(range);
-}
-
-void BackgroundParser::rangeContentsChanged(KTextEditor::SmartRange* range, KTextEditor::SmartRange* mostSpecificChild)
+QList< IndexedString > BackgroundParser::managedDocuments()
 {
     QMutexLocker l(&d->m_mutex);
-
-    // Smart mutex is already locked
-    KUrl documentUrl = range->document()->url();
-
-    ///@todo Also detect these changes on-disk and do clearing
-    ModificationRevisionSet::clearCache();
-    KDevelop::ModificationRevision::clearModificationCache(IndexedString(documentUrl));
-
-    if (d->m_parseJobs.contains(documentUrl)) {
-        ParseJob* job = d->m_parseJobs[documentUrl];
-        if (job->addChangedRange( mostSpecificChild ))
-            // Success
-            return;
-    }
-
-    // Initially I just created a new parse job here, but that causes a deadlock as the smart mutex is locked
-    // So store the info in a class with just the changed ranges information...
-    DocumentChangeTracker* newTracker = 0;
-    if (d->m_delayedParseJobs.contains(documentUrl))
-        newTracker = d->m_delayedParseJobs[documentUrl];
-
-    if (!newTracker) {
-        newTracker = new DocumentChangeTracker();
-        d->m_delayedParseJobs.insert(documentUrl, newTracker);
-    }
-
-    newTracker->addChangedRange( mostSpecificChild );
     
-    d->startTimerThreadSafe();
+    return d->m_managed.keys();
+}
+
+DocumentChangeTracker* BackgroundParser::trackerForUrl(const KDevelop::IndexedString& url) const
+{
+    QMutexLocker l(&d->m_mutex);
+    
+    QHash< IndexedString, DocumentChangeTracker* >::iterator it = d->m_managed.find(url);
+    if(it != d->m_managed.end())
+        return *it;
+    else
+        return 0;
+}
+
+void BackgroundParser::documentClosed ( IDocument* document )
+{
+    QMutexLocker l(&d->m_mutex);
+    
+    if(document->textDocument())
+    {
+        KTextEditor::Document* textDocument = document->textDocument();
+        
+        if(!d->m_managedTextDocumentUrls.contains(textDocument))
+            return; // Probably the document had an invalid url, and thus it wasn't added to the background parser
+        
+        Q_ASSERT(d->m_managedTextDocumentUrls.contains(textDocument));
+        
+        IndexedString url(d->m_managedTextDocumentUrls[textDocument]);
+        Q_ASSERT(d->m_managed.contains(url));
+        
+        kDebug() << "removing" << url.str() << "from background parser";
+        delete d->m_managed[url];
+        d->m_managedTextDocumentUrls.remove(textDocument);
+        d->m_managed.remove(url);
+    }
+}
+
+void BackgroundParser::documentLoaded( IDocument* document )
+{
+    QMutexLocker l(&d->m_mutex);
+    if(document->textDocument() && document->textDocument()->url().isValid())
+    {
+        KTextEditor::Document* textDocument = document->textDocument();
+        
+        IndexedString url(document->url());
+        // Some debugging because we had issues with this
+        
+        if(d->m_managed.contains(url) && d->m_managed[url]->document() == textDocument)
+        {
+            kDebug() << "Got redundant documentLoaded from" << document->url() << textDocument;
+            return;
+        }
+        
+        kDebug() << "Creating change tracker for " << document->url();
+        
+        
+        Q_ASSERT(!d->m_managed.contains(url));
+        Q_ASSERT(!d->m_managedTextDocumentUrls.contains(textDocument));
+        
+        d->m_managedTextDocumentUrls[textDocument] = url;
+        d->m_managed.insert(url, new DocumentChangeTracker(textDocument));
+    }else{
+        kDebug() << "NOT creating change tracker for" << document->url();
+    }
+}
+
+void BackgroundParser::documentUrlChanged(IDocument* document)
+{
+    documentClosed(document);
+    
+    // Only call documentLoaded if the file wasn't renamed to a filename that is already tracked.
+    if(document->textDocument() && !d->m_managed.contains(IndexedString(document->textDocument()->url())))
+        documentLoaded(document);
 }
 
 void BackgroundParser::startTimer() {
-    if (!d->m_timer.isActive())
-        d->m_timer.start(d->m_delay);
+    d->m_timer.start(d->m_delay);
 }
 
-
 }
 
 #include "backgroundparser.moc"
