@@ -22,7 +22,7 @@
 
 #include <QtCore/QList>
 #include <QtCore/QMap>
-#include <QtCore/qcontainerfwd.h>
+//#include <QtCore/qcontainerfwd.h>
 
 #include "../editor/documentrangeobject.h"
 #include "identifier.h"
@@ -34,10 +34,6 @@
 
 class QByteArray;
 
-namespace KTextEditor {
-  class SmartRange;
-}
-
 namespace KDevelop
 {
 
@@ -65,7 +61,8 @@ public:
   enum AccessPolicy {
     Public    /**< a public declaration */,
     Protected /**< a protected declaration */,
-    Private   /**< a private declaration */
+    Private   /**< a private declaration */,
+    DefaultAccess /**<a declaration with default access; in java, only package-level access. */
   };
   /// Enumeration of the types of declarations
   enum Kind {
@@ -73,7 +70,8 @@ public:
     Instance /**< An instance of a type is declared("MyClass m;") */,
     NamespaceAlias/**< This is a namespace-alias. You can safely cast this object to NamespaceAliasDeclaration. */,
     Alias, /**<This is an alias-declaration. You can safely cast this object to AliasDeclaration. */
-    Namespace /**< Declaration of a namespace. */
+    Namespace, /**< Declaration of a namespace. */
+    Import /**< Declaration marks the Import of a file. */
   };
 
   /**
@@ -85,7 +83,7 @@ public:
    * \param range range of the alias declaration's identifier
    * \param parentContext context in which this declaration occurred
    * */
-  Declaration(const SimpleRange& range, DUContext* parentContext);
+  Declaration(const RangeInRevision& range, DUContext* parentContext);
   ///Copy-constructor for cloning
   Declaration(const Declaration& rhs);
   /// Destructor
@@ -126,15 +124,21 @@ public:
   /// Set whether this declaration is a type alias. \param typeAlias true if the declaration is a type alias, otherwise false.
   void setIsTypeAlias(bool typeAlias);
 
+  /// Access whether the declaration is final, ie, it can only be written to once [Java]
+  bool isFinal() const;
+  /// Set whether the declaration is final. \param final true if the declaration is final, otherwise false.
+  void setFinal(bool final);
+
   /**
    * Retrieve the declaration which is specialized with the given \a specialization index as seen from \a topContext.
    *
    * \param specialization the specialization index (see DeclarationId)
-   * \param topContext the top context to search from
+   * \param topContext the top context representing the perspective from which to specialize.
+   *                                if @p topContext is zero, only already existing specializations are returned, and if none exists, zero is returned.
    * \param upDistance upwards distance in the context-structure of the
    *                   given specialization-info. This allows specializing children.
    * */
-  virtual Declaration* specialize(uint specialization, const TopDUContext* topContext, int upDistance = 0);
+  virtual Declaration* specialize(IndexedInstantiationInformation specialization, const TopDUContext* topContext, int upDistance = 0);
 
   /**
    * Retrieve the context that is opened by this declaration, if one exists.
@@ -257,7 +261,7 @@ public:
    * \return this declaration's identifier in indexed form. This is faster than identifier(), because it
    *         equals the internal representation. Use this for example to do equality-comparison.
    */
-  IndexedIdentifier indexedIdentifier() const;
+  const IndexedIdentifier& indexedIdentifier() const;
 
   /**
    * Determine the global qualified identifier of this declaration.
@@ -313,7 +317,7 @@ public:
    *
    * \param inSymbolTable true to add this declaration to the symbol table, false to remove it.
    */
-  void setInSymbolTable(bool inSymbolTable);
+  virtual void setInSymbolTable(bool inSymbolTable);
 
   /**
    * Equivalence operator.
@@ -327,23 +331,27 @@ public:
    */
   virtual QString toString() const;
 
-  ///@todo The following two are convenience-functions. Think whether they should stay here, or be moved out.
+  /**
+   * Returns a list of pairs:
+   * An url of a file, paired together with all use-ranges of this declaration in that file.
+   * The ranges are in the documents local revision (use DUChainUtils::transformFromRevision or usesCurrentRevision())
+   * The uses are unique, no 2 uses are returend that have the same range within the same file.
+   * 
+   * This is a non-trivial operation.
+   * */
+  QMap<IndexedString, QList<RangeInRevision> > uses() const;
 
   /**
    * Returns a list of pairs:
    * An url of a file, paired together with all use-ranges of this declaration in that file.
+   * The ranges are in the most current document revisions available.
    * The uses are unique, no 2 uses are returend that have the same range within the same file.
-   *
+   * 
+   * @warning This must be called only from within the foreground, or with the foreground lock locked.
    * This is a non-trivial operation.
    * */
-  QMap<IndexedString, QList<SimpleRange> > uses() const;
-
-  /**
-   * Collects the smart-ranges for all uses. Uses that do not have smart ranges are not represented
-   * in the result.
-   * */
-  QList<KTextEditor::SmartRange*> smartUses() const;
-
+  QMap<IndexedString, QList<SimpleRange> > usesCurrentRevision() const;
+  
   /**
     * This hash-value should differentiate between multiple different
     * declarations that have the same qualifiedIdentifier, but should have a different
@@ -356,7 +364,7 @@ public:
   /**
    *
    * */
-  virtual uint specialization() const;
+  virtual IndexedInstantiationInformation specialization() const;
 
   /**
    * @see DeclarationId
@@ -377,7 +385,7 @@ public:
 
   ///Whether this declaration has been inserted anonymously into its parent-context
   bool isAnonymous() const;
-  
+
   /**
    * Clear the index for this declaration in the top context that was allocated with allocateOwnIndex().
    */
@@ -419,8 +427,16 @@ protected:
     * \param url document url in which this object is located.
     * \param range text range which this object covers.
     */
-  Declaration( DeclarationData & dd, const SimpleRange& range );
+  Declaration( DeclarationData & dd, const RangeInRevision& range );
 
+  /**
+   * Returns true if this declaration is being currently destroyed persistently,
+   * which means that it should eventually deregister itself from persistent storage facilities.
+   *
+   * Only call this from destructors.
+   */
+  bool persistentlyDestroying() const;
+  
   DUCHAIN_DECLARE_DATA(Declaration)
 private:
    /**
