@@ -22,15 +22,16 @@
 #include "outputwidget.h"
 
 #include "standardoutputview.h"
-#include <QtCore/QSignalMapper>
 #include <QtCore/QTimer>
 #include <QtGui/QAbstractItemDelegate>
 #include <QtGui/QItemSelectionModel>
-#include <QtGui/QListView>
+#include <QtGui/QTreeView>
 #include <QtGui/QToolButton>
 #include <QtGui/QScrollBar>
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QStackedWidget>
+#include <QtGui/QApplication>
+#include <QtGui/QClipboard>
 #include <kmenu.h>
 #include <kaction.h>
 #include <kdebug.h>
@@ -38,18 +39,18 @@
 #include <klocale.h>
 #include <kicon.h>
 #include <ktabwidget.h>
+#include <kstandardaction.h>
 
 #include <outputview/ioutputviewmodel.h>
+#include <util/focusedtreeview.h>
 
 #include "toolviewdata.h"
 
 OutputWidget::OutputWidget(QWidget* parent, ToolViewData* tvdata)
-    : QWidget( parent ), tabwidget(0), data(tvdata), scrollModelViewMapper(new QSignalMapper(this)), scrollModelViewMapper2(new QSignalMapper(this))
+    : QWidget( parent ), tabwidget(0), data(tvdata)
 {
     setWindowTitle(i18n("Output View"));
     setWindowIcon(tvdata->icon);
-    connect(scrollModelViewMapper, SIGNAL(mapped(int)), this, SLOT(scrollToBottom(int)));
-    connect(scrollModelViewMapper2, SIGNAL(mapped(int)), this, SLOT(doScrollToBottom(int)));
     QVBoxLayout* layout = new QVBoxLayout(this);
     layout->setMargin(0);
     if( data->type & KDevelop::IOutputView::MultipleView )
@@ -83,6 +84,18 @@ OutputWidget::OutputWidget(QWidget* parent, ToolViewData* tvdata)
     addAction(focusOnSelect);
     focusOnSelect->setChecked( false );
 
+    QAction *separator = new QAction(this);
+    separator->setSeparator(true);
+    addAction(separator);
+    
+    KAction *selectAllAction = KStandardAction::selectAll(this, SLOT(selectAll()), this);
+    selectAllAction->setShortcut(KShortcut()); //FIXME: why does CTRL-A conflict with Katepart (while CTRL-Cbelow doesn't) ?
+    selectAllAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    addAction(selectAllAction);
+
+    KAction *copyAction = KStandardAction::copy(this, SLOT(copySelection()), this);
+    copyAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    addAction(copyAction);
 
     connect( data, SIGNAL( outputAdded( int ) ),
              this, SLOT( addOutput( int ) ) );
@@ -103,14 +116,16 @@ OutputWidget::OutputWidget(QWidget* parent, ToolViewData* tvdata)
 
 void OutputWidget::addOutput( int id )
 {
-    QListView* listview = createListView(id);
+    QTreeView* listview = createListView(id);
+    listview->setSelectionMode(QAbstractItemView::ContiguousSelection);
     setCurrentWidget( listview );
     connect( data->outputdata.value(id), SIGNAL(modelChanged(int)), this, SLOT(changeModel(int)));
     connect( data->outputdata.value(id), SIGNAL(delegateChanged(int)), this, SLOT(changeDelegate(int)));
+    
     enableActions();
 }
 
-void OutputWidget::setCurrentWidget( QListView* view )
+void OutputWidget::setCurrentWidget( QTreeView* view )
 {
     if( data->type & KDevelop::IOutputView::MultipleView )
     {
@@ -134,13 +149,17 @@ void OutputWidget::changeModel( int id )
     if( data->outputdata.contains( id ) && views.contains( id ) )
     {
         OutputData* od = data->outputdata.value(id);
-        scrollModelViewMapper->removeMappings( views.value( id )->model() );
         views.value( id )->setModel(od->model);
-        if( od->behaviour & KDevelop::IOutputView::AutoScroll && od->model )
+
+        if (!od->model)
+            return;
+
+        disconnect( od->model,SIGNAL(rowsInserted(const QModelIndex&, int, int)), this,
+                    SLOT(rowsInserted(const QModelIndex&, int, int)) );
+        if( od->behaviour & KDevelop::IOutputView::AutoScroll )
         {
-            scrollModelViewMapper->setMapping( od->model, id );
             connect( od->model,SIGNAL(rowsInserted(const QModelIndex&, int, int)),
-                     scrollModelViewMapper, SLOT(map()) );
+                     SLOT(rowsInserted(const QModelIndex&, int, int)) );
         }
     }
     else
@@ -155,7 +174,7 @@ void OutputWidget::removeOutput( int id )
     {
         if( data->type & KDevelop::IOutputView::MultipleView || data->type & KDevelop::IOutputView::HistoryView )
         {
-            QListView* w = views.value(id);
+            QTreeView* w = views.value(id);
             if( data->type & KDevelop::IOutputView::MultipleView )
             {
                 int idx = tabwidget->indexOf( w );
@@ -168,12 +187,15 @@ void OutputWidget::removeOutput( int id )
                 stackwidget->removeWidget( w );
             }
             delete w;
+            views.remove( id );
         } else
         {
             views.value( id )->setModel( 0 );
             views.value( id )->setItemDelegate( 0 );
         }
-        scrollModelViewMapper->removeMappings( data->outputdata.value( id )->model );
+        disconnect( data->outputdata.value( id )->model,SIGNAL(rowsInserted(const QModelIndex&, int, int)),
+                    this, SLOT(rowsInserted(const QModelIndex&, int, int)) );
+        
         emit outputRemoved( data->toolViewId, id );
     }
     enableActions();
@@ -297,19 +319,20 @@ void OutputWidget::activate(const QModelIndex& index)
     }
 }
 
-QListView* OutputWidget::createListView(int id)
+QTreeView* OutputWidget::createListView(int id)
 {
-    QListView* listview = 0;
+    QTreeView* listview = 0;
     if( !views.contains(id) )
     {
         if( data->type & KDevelop::IOutputView::MultipleView || data->type & KDevelop::IOutputView::HistoryView )
         {
             kDebug() << "creating listview";
-            listview = new QListView(this);
+            listview = new KDevelop::FocusedTreeView(this);
             listview->setEditTriggers( QAbstractItemView::NoEditTriggers );
-            listview->setViewMode( QListView::ListMode );
-            listview->setMovement( QListView::Static );
-            listview->setResizeMode( QListView::Fixed );
+            listview->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn); //Always enable the scrollbar, so it doesn't flash around
+            listview->setHeaderHidden(true);
+            listview->setRootIsDecorated(false);
+            
             views[id] = listview;
             connect( listview, SIGNAL(activated(const QModelIndex&)),
                      this, SLOT(activate(const QModelIndex&)));
@@ -328,10 +351,12 @@ QListView* OutputWidget::createListView(int id)
         {
             if( views.isEmpty() )
             {
-                listview = new QListView(this);
+                listview = new KDevelop::FocusedTreeView(this);
                 listview->setEditTriggers( QAbstractItemView::NoEditTriggers );
-                listview->setViewMode( QListView::ListMode );
-                listview->setUniformItemSizes( true );
+                listview->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn); //Always enable the scrollbar, so it doesn't flash around
+                listview->setRootIsDecorated(false);
+                listview->setHeaderHidden(true);
+
                 layout()->addWidget( listview );
                 connect( listview, SIGNAL(activated(const QModelIndex&)),
                          this, SLOT(activate(const QModelIndex&)));
@@ -406,31 +431,15 @@ void OutputWidget::enableActions()
     }
 }
 
-void OutputWidget::scrollToBottom( int id )
-{
-    if( views.contains( id ) && views.value( id )->verticalScrollBar()->value() == views.value( id )->verticalScrollBar()->maximum() )
-    {
-        kDebug() << "starting auto-scroll timer";
-        if (!scrollTimers.contains(id)) {
-            QTimer* timer = new QTimer(this);
-            timer->setSingleShot( true );
-            scrollTimers[id] = timer;
-            scrollModelViewMapper2->setMapping(timer, id);
-            connect(timer, SIGNAL(timeout()), scrollModelViewMapper2, SLOT(map()));
+void OutputWidget::rowsInserted(const QModelIndex& parent, int from, int to) {
+    Q_UNUSED(parent);
+    for( QMap< int, QTreeView* >::const_iterator it = views.constBegin(); it != views.constEnd(); ++it) {
+        if((*it)->model() == sender()) {
+            QModelIndex pre = (*it)->model()->index(from-1, 0);
+            if(!pre.isValid() || ((*it)->visualRect(pre).isValid() && (*it)->viewport()->rect().intersects((*it)->visualRect(pre)) && to == (*it)->model()->rowCount()-1)) {
+                (*it)->scrollToBottom();
+            }
         }
-
-        if (!scrollTimers[id]->isActive())
-            // 100ms delay in scrolling, for performance reasons
-            scrollTimers[id]->start(100);
-    }
-}
-
-void OutputWidget::doScrollToBottom( int id )
-{
-    if( views.contains( id ) )
-    {
-        kDebug() << "doing auto-scroll";
-        views.value( id )->scrollToBottom();
     }
 }
 
@@ -439,9 +448,40 @@ void OutputWidget::scrollToIndex( const QModelIndex& idx )
     QWidget* w = currentWidget();
     if( !w )
         return;
-    kDebug() << "scrolling to index:" << idx;
     QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(w);
     view->scrollTo( idx );
 }
 
+void OutputWidget::copySelection()
+{
+    QWidget* widget = currentWidget();
+    if( !widget )
+        return;
+    QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(widget);
+    if( !view )
+        return;
+
+    QClipboard *cb = QApplication::clipboard();
+    QModelIndexList indexes = view->selectionModel()->selectedRows();
+    QString content;
+    Q_FOREACH( QModelIndex index, indexes) {
+      content += view->model()->data(index).toString() + "\n";
+    }
+    cb->setText(content);
+}
+
+void OutputWidget::selectAll()
+{
+    QWidget* widget = currentWidget();
+    if( !widget )
+        return;
+    QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(widget);
+    if( !view )
+        return;
+
+    view->selectAll();
+}
+
+
+
 #include "outputwidget.moc"
