@@ -31,7 +31,7 @@ Boston, MA 02110-1301, USA.
 #include <QApplication>
 
 #include <kio/netaccess.h>
-#include <kfiledialog.h>
+#include <kencodingfiledialog.h>
 #include <kactioncollection.h>
 #include <klocale.h>
 #include <krecentfilesaction.h>
@@ -51,14 +51,25 @@ Boston, MA 02110-1301, USA.
 #include "uicontroller.h"
 #include "partcontroller.h"
 #include "savedialog.h"
+#include <kmessagebox.h>
+#include <KIO/Job>
 
+#include <config-kdevplatform.h>
+#if HAVE_KOMPARE
+    #include "patchdocument.h"
+#endif
+
+#define EMPTY_DOCUMENT_URL i18n("Untitled")
 
 namespace KDevelop
 {
 
+
 struct DocumentControllerPrivate {
     DocumentControllerPrivate(DocumentController* c)
-    : controller(c)
+        : controller(c)
+        , fileOpenRecent(0)
+        , globalTextEditorInstance(0)
     {
     }
 
@@ -98,7 +109,27 @@ struct DocumentControllerPrivate {
     }
     void chooseDocument()
     {
-        controller->openDocument(KUrl());
+
+        KUrl dir;
+        if( controller->activeDocument() )
+        {
+            dir = KUrl( controller->activeDocument()->url() );
+            dir.setFileName(QString());
+        }else
+        {
+            dir = KGlobal::config()->group("Open File").readEntry( "Last Open File Directory", Core::self()->projectController()->projectsBaseDirectory() );
+        }
+
+        KEncodingFileDialog::Result res = KEncodingFileDialog::getOpenUrlsAndEncoding( controller->encoding(), dir.url(), i18n( "*|Text File\n" ),
+                                    Core::self()->uiControllerInternal()->defaultMainWindow(),
+                                    i18n( "Open File" ) );
+        if( !res.URLs.isEmpty() ) {
+            QString encoding = res.encoding;
+            foreach( const KUrl& u, res.URLs ) {
+                openDocumentInternal(u, QString(), KTextEditor::Range::invalid(), encoding  );
+            }
+        }
+        
     }
 
     void changeDocumentUrl(KDevelop::IDocument* document)
@@ -128,6 +159,254 @@ struct DocumentControllerPrivate {
         }
     }
 
+    IDocument* openDocumentInternal( const KUrl & inputUrl, const QString& prefName = QString(),
+        const KTextEditor::Range& range = KTextEditor::Range::invalid(), const QString& encoding = "",
+        DocumentController::DocumentActivationParams activationParams = 0 )
+    {
+        IDocument* previousActiveDocument = controller->activeDocument();
+        KTextEditor::Cursor previousActivePosition;
+        if(previousActiveDocument && previousActiveDocument->textDocument() && previousActiveDocument->textDocument()->activeView())
+            previousActivePosition = previousActiveDocument->textDocument()->activeView()->cursorPosition();
+        
+
+        QString _encoding = encoding;
+        
+        KUrl url = inputUrl;
+
+        if ( url.isEmpty() && (!activationParams.testFlag(IDocumentController::DoNotCreateView)) )
+        {
+            KUrl dir;
+            if( controller->activeDocument() )
+            {
+                dir = controller->activeDocument()->url().upUrl();
+            }else
+            {
+                dir = KGlobal::config()->group("Open File").readEntry( "Last Open File Directory", Core::self()->projectController()->projectsBaseDirectory() );
+            }
+
+            KEncodingFileDialog::Result res = KEncodingFileDialog::getOpenUrlAndEncoding( "", dir.url(), i18n( "*|Text File\n" ),
+                                        Core::self()->uiControllerInternal()->defaultMainWindow(),
+                                        i18n( "Open File" ) );
+            if( !res.URLs.isEmpty() )
+                url = res.URLs.first();
+            _encoding = res.encoding;
+        }
+        if ( url.isEmpty() )
+            //still no url
+            return 0;
+
+        KGlobal::config()->group("Open File").writeEntry( "Last Open File Directory", url.upUrl() );
+
+        // clean it and resolve possible symlink
+        url.cleanPath( KUrl::SimplifyDirSeparators );
+        if ( url.isLocalFile() )
+        {
+            QString path = QFileInfo( url.toLocalFile() ).canonicalFilePath();
+            if ( !path.isEmpty() )
+                url.setPath( path );
+        }
+        
+        //get a part document
+        IDocument* doc=0;
+        if (documents.contains(url))
+            doc=documents.value(url);
+        else
+        {
+            KMimeType::Ptr mimeType;
+
+            if (DocumentController::isEmptyDocumentUrl(url))
+            {
+                mimeType = KMimeType::mimeType("text/plain");
+            }
+            else
+            {
+                //make sure the URL exists
+                if ( !url.isValid() || !KIO::NetAccess::exists( url, KIO::NetAccess::DestinationSide, 0 ) )
+                {
+                    kDebug() << "cannot find URL:" << url.url();
+                    return 0;
+                }
+
+                mimeType = KMimeType::findByUrl( url );
+                
+                if( !url.isLocalFile() && mimeType->isDefault() )
+                {
+                    // fall back to text/plain, for remote files without extension, i.e. COPYING, LICENSE, ...
+                    // using a syncronous KIO::MimetypeJob is hazardous and may lead to repeated calls to
+                    // this function without it having returned in the first place
+                    // and this function is *not* reentrant, see assert below:
+                    // Q_ASSERT(!documents.contains(url) || documents[url]==doc);
+                    mimeType = KMimeType::mimeType("text/plain");
+                }
+            }
+
+            // is the URL pointing to a directory?
+            if ( mimeType->is( "inode/directory" ) )
+            {
+                kDebug() << "cannot open directory:" << url.url();
+                return 0;
+            }
+
+            if( prefName.isEmpty() )
+            {
+                // Try to find a plugin that handles this mimetype
+                QString constraint = QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimeType->name());
+                KPluginInfo::List plugins = IPluginController::queryPlugins( constraint );
+
+                if( !plugins.isEmpty() )
+                {
+                    KPluginInfo info = plugins.first();
+                    Core::self()->pluginController()->loadPlugin( info.pluginName() );
+                }
+            }
+            
+            if( factories.contains( mimeType->name() ) )
+            {
+                doc = factories[mimeType->name()]->create(url, Core::self());
+            }
+            
+            if(!doc) {
+                if( !prefName.isEmpty() ) 
+                {
+                    doc = new PartDocument(url, Core::self(), prefName);
+                } else  if ( Core::self()->partControllerInternal()->isTextType(mimeType)) 
+                {
+                    doc = new TextDocument(url, Core::self(), _encoding);
+                } else if( Core::self()->partControllerInternal()->canCreatePart(url) ) 
+                {
+                    doc = new PartDocument(url, Core::self());
+                } else
+                {
+                    int openAsText = KMessageBox::questionYesNo(0, i18n("KDevelop could not find the editor for file '%1' of type %2.\nDo you want to open it as plain text?", url.fileName(), mimeType->name()), i18n("Could Not Find Editor"));
+                    if (openAsText == KMessageBox::Yes)
+                        doc = new TextDocument(url, Core::self(), _encoding);
+                    else
+                        return 0;
+                }
+            }
+        }
+        
+        // The url in the document must equal the current url, else the housekeeping will get broken
+        Q_ASSERT(!doc || doc->url() == url);
+        
+        if(doc && openDocumentInternal(doc, range, activationParams))
+            return doc;
+        else
+            return 0;
+        
+    }
+    
+    bool openDocumentInternal(IDocument* doc,
+                                const KTextEditor::Range& range,
+                                DocumentController::DocumentActivationParams activationParams)
+    {
+        IDocument* previousActiveDocument = controller->activeDocument();
+        KTextEditor::Cursor previousActivePosition;
+        if(previousActiveDocument && previousActiveDocument->textDocument() && previousActiveDocument->textDocument()->activeView())
+            previousActivePosition = previousActiveDocument->textDocument()->activeView()->cursorPosition();
+        
+        KUrl url=doc->url();
+        UiController *uiController = Core::self()->uiControllerInternal();
+        Sublime::Area *area = uiController->activeArea();
+        
+        //We can't have the same url in many documents
+        //so we check it's already the same if it exists
+        //contains=>it's the same
+        Q_ASSERT(!documents.contains(url) || documents[url]==doc);
+
+        Sublime::Document *sdoc = dynamic_cast<Sublime::Document*>(doc);
+        if( !sdoc )
+        {
+            documents.remove(url);
+            delete doc;
+            return false;
+        }
+        //react on document deletion - we need to cleanup controller structures
+        QObject::connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), controller, SLOT(removeDocument(Sublime::Document*)));
+        //We check if it was already opened before
+        bool emitOpened = !documents.contains(url);
+        if(emitOpened)
+            documents[url]=doc;
+        
+        if (!activationParams.testFlag(IDocumentController::DoNotCreateView))
+        {
+            //find a view if there's one already opened in this area
+            Sublime::View *partView = 0;
+            foreach (Sublime::View *view, sdoc->views())
+            {
+                if (area->views().contains(view) && (activationParams.testFlag(DocumentController::DoNotForceCurrentView) || area->indexOf(view) == area->indexOf(uiController->activeSublimeWindow()->activeView())))
+                {
+                    partView = view;
+                    break;
+                }
+            }
+            bool addView = false, applyRange = true;
+            if (!partView)
+            {
+                //no view currently shown for this url
+                partView = sdoc->createView();
+                addView = true;
+            }
+            
+            KDevelop::TextView* textView = dynamic_cast<KDevelop::TextView*>(partView);
+            if(textView && textView->textView()) {
+                applyRange = false;
+                if (range.isEmpty())
+                    doc->setCursorPosition( range.start() );
+                else
+                    doc->setTextSelection( range );
+            }else if(textView) {
+                textView->setInitialRange(range);
+            }
+            
+            if(addView) {
+                //add view to the area
+                area->addView(partView, uiController->activeSublimeWindow()->activeView());
+            }
+            
+            if (!activationParams.testFlag(IDocumentController::DoNotActivate))
+            {
+                uiController->activeSublimeWindow()->activateView(partView);
+            }
+            fileOpenRecent->addUrl( url );
+
+            if( applyRange && range.isValid() )
+            {
+                if (range.isEmpty())
+                    doc->setCursorPosition( range.start() );
+                else
+                    doc->setTextSelection( range );
+            }
+        }
+
+        // Deferred signals, wait until it's all ready first
+        if( emitOpened ) {
+            emit controller->documentOpened( doc );
+        }
+
+        if (!activationParams.testFlag(IDocumentController::DoNotActivate) && doc != controller->activeDocument())
+            emit controller->documentActivated( doc );
+
+        saveAll->setEnabled(true);
+        revertAll->setEnabled(true);
+        close->setEnabled(true);
+        closeAll->setEnabled(true);
+        closeAllOthers->setEnabled(true);
+
+        if(doc) {
+            KTextEditor::Cursor activePosition;
+            if(range.isValid())
+                activePosition = range.start();
+            else if(doc->textDocument() && doc->textDocument()->activeView())
+                activePosition = doc->textDocument()->activeView()->cursorPosition();
+
+            if (doc != previousActiveDocument || activePosition != previousActivePosition)
+                emit controller->documentJumpPerformed(doc, activePosition, previousActiveDocument, previousActivePosition);
+        }
+        
+        return true;
+    }
+
     DocumentController* controller;
 
     QList<HistoryEntry> backHistory;
@@ -140,6 +419,7 @@ struct DocumentControllerPrivate {
     QPointer<KAction> closeAll;
     QPointer<KAction> closeAllOthers;
     KRecentFilesAction* fileOpenRecent;
+    KTextEditor::Document* globalTextEditorInstance;
 
 /*    HistoryEntry createHistoryEntry();
     void addHistoryEntry();
@@ -158,6 +438,10 @@ DocumentController::DocumentController( QObject *parent )
     connect(this, SIGNAL(documentUrlChanged(KDevelop::IDocument*)), this, SLOT(changeDocumentUrl(KDevelop::IDocument*)));
 
     if(!(Core::self()->setupFlags() & Core::NoUi)) setupActions();
+
+#if HAVE_KOMPARE    
+    registerDocumentForMimetype("text/x-patch", new PatchDocumentFactory);
+#endif
 }
 
 void KDevelop::DocumentController::initialize()
@@ -166,7 +450,8 @@ void KDevelop::DocumentController::initialize()
 
 void DocumentController::cleanup()
 {
-    d->fileOpenRecent->saveEntries( KConfigGroup(KGlobal::config(), "Recent Files" ) );
+    if (d->fileOpenRecent)
+        d->fileOpenRecent->saveEntries( KConfigGroup(KGlobal::config(), "Recent Files" ) );
 
     // Close all documents without checking if they should be saved.
     // This is because the user gets a chance to save them during MainWindow::queryClose.
@@ -190,7 +475,7 @@ void DocumentController::setupActions()
     action = ac->addAction( "file_open" );
     action->setIcon(KIcon("document-open"));
     action->setShortcut( Qt::CTRL + Qt::Key_O );
-    action->setText(i18n( "&Open File..." ) );
+    action->setText(i18n( "&Open..." ) );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( chooseDocument() ) );
     action->setToolTip( i18n( "Open file" ) );
     action->setWhatsThis( i18n( "<b>Open file</b><p>Opens a file for editing.</p>" ) );
@@ -219,7 +504,7 @@ void DocumentController::setupActions()
     action = d->close = ac->addAction( "file_close" );
     action->setIcon(KIcon("window-close"));
     action->setShortcut( Qt::CTRL + Qt::Key_W );
-    action->setText( i18n( "&Close File" ) );
+    action->setText( i18n( "&Close" ) );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( fileClose() ) );
     action->setToolTip( i18n( "Close File" ) );
     action->setWhatsThis( i18n( "<b>Close File</b><p>Closes current file.</p>" ) );
@@ -235,6 +520,7 @@ void DocumentController::setupActions()
 
     action = d->closeAllOthers = ac->addAction( "file_closeother" );
     action->setIcon(KIcon("window-close"));
+    action->setShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_W );
     action->setText(i18n( "Close All Ot&hers" ) );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( closeAllOtherDocuments() ) );
     action->setToolTip( i18n( "Close all other documents" ) );
@@ -259,191 +545,33 @@ void DocumentController::slotOpenDocument(const KUrl &url)
 
 IDocument* DocumentController::openDocumentFromText( const QString& data )
 {
-    KTemporaryFile *temp = new KTemporaryFile();
-    temp->setSuffix("kdevtmp");
-    temp->open();
-    temp->write(data.toUtf8());
-    temp->flush();
-    d->tempFiles << temp;
-    return openDocument(temp->fileName());
+    IDocument* d = openDocument(nextEmptyDocumentUrl());
+    d->textDocument()->setText( data );
+    return d;
+}
+
+IDocument* DocumentController::openDocument( const KUrl& inputUrl, const QString& prefName )
+{
+    return d->openDocumentInternal( inputUrl, prefName );
 }
 
 IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         const KTextEditor::Range& range,
-        DocumentActivationParams activationParams)
+        DocumentActivationParams activationParams,
+        const QString& encoding)
 {
-    IDocument* previousActiveDocument = activeDocument();
-    KTextEditor::Cursor previousActivePosition;
-    if(previousActiveDocument && previousActiveDocument->textDocument() && previousActiveDocument->textDocument()->activeView())
-        previousActivePosition = previousActiveDocument->textDocument()->activeView()->cursorPosition();
-    
-
-    UiController *uiController = Core::self()->uiControllerInternal();
-    Sublime::Area *area = uiController->activeArea();
-
-    KUrl url = inputUrl;
-
-    if ( url.isEmpty() && (!activationParams.testFlag(IDocumentController::DoNotCreateView)) )
-    {
-        KUrl dir;
-        if( activeDocument() )
-        {
-            dir = KUrl( activeDocument()->url().directory() );
-        }else
-        {
-            dir = Core::self()->projectController()->projectsBaseDirectory();
-        }
-
-        url = KFileDialog::getOpenUrl( dir, i18n( "*.*|Text File\n" ),
-                                       Core::self()->uiControllerInternal()->defaultMainWindow(),
-                                       i18n( "Open File" ) );
-    }
-    if ( url.isEmpty() )
-        //still no url
-        return 0;
-
-    bool emitOpened = false;
-
-    //get a part document
-    if (!d->documents.contains(url))
-    {
-        //make sure the URL exists
-        if ( !url.isValid() || !KIO::NetAccess::exists( url, KIO::NetAccess::DestinationSide, 0 ) )
-        {
-            kDebug() << "cannot find URL:" << url.url();
-            return 0;
-        }
-
-        // clean it and resolve possible symlink
-        url.cleanPath( KUrl::SimplifyDirSeparators );
-        if ( url.isLocalFile() )
-        {
-            QString path = QFileInfo( url.toLocalFile() ).canonicalFilePath();
-            if ( !path.isEmpty() )
-                url.setPath( path );
-        }
-
-        KMimeType::Ptr mimeType = KMimeType::findByUrl( url );
-
-        // is the URL pointing to a directory?
-        if ( mimeType->is( "inode/directory" ) )
-        {
-            kDebug() << "cannot open directory:" << url.url();
-            return 0;
-        }
-
-        // Try to find a plugin that handles this mimetype
-        QString constraint = QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimeType->name());
-        KPluginInfo::List plugins = IPluginController::queryPlugins( constraint );
-
-        if( !plugins.isEmpty() )
-        {
-            KPluginInfo info = plugins.first();
-            kDebug() << "loading" << info.pluginName();
-            Core::self()->pluginController()->loadPlugin( info.pluginName() );
-            if( d->factories.contains( mimeType->name() ) )
-            {
-                IDocument* idoc = d->factories[mimeType->name()]->create(url, Core::self());
-                if( idoc )
-                {
-                     d->documents[url] = idoc;
-                }
-            }
-        }
-        if ( !d->documents.contains(url) && Core::self()->partControllerInternal()->isTextType(mimeType))
-            d->documents[url] = new TextDocument(url, Core::self());
-        else if( !d->documents.contains(url) )
-            d->documents[url] = new PartDocument(url, Core::self());
-        emitOpened = d->documents.contains(url);
-    }
-    IDocument *doc = d->documents[url];
-
-    Sublime::Document *sdoc = dynamic_cast<Sublime::Document*>(doc);
-    if( !sdoc )
-    {
-        d->documents.remove(url);
-        delete doc;
-        return 0;
-    }
-    //react on document deletion - we need to cleanup controller structures
-    connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), this, SLOT(removeDocument(Sublime::Document*)));
-
-    if (!activationParams.testFlag(IDocumentController::DoNotCreateView))
-    {
-        //find a view if there's one already opened in this area
-        Sublime::View *partView = 0;
-        foreach (Sublime::View *view, sdoc->views())
-        {
-            if (area->views().contains(view))
-            {
-                partView = view;
-                break;
-            }
-        }
-        bool addView = false, applyRange = true;
-        if (!partView)
-        {
-            //no view currently shown for this url
-            partView = sdoc->createView();
-            addView = true;
-        }
-        
-        KDevelop::TextView* textView = dynamic_cast<KDevelop::TextView*>(partView);
-        if(textView && textView->textView()) {
-            applyRange = false;
-            if (range.isEmpty())
-                textView->textView()->setCursorPosition( range.start() );
-            else
-                textView->textView()->setSelection( range );
-        }else if(textView) {
-            textView->setInitialRange(range);
-        }
-        
-        if(addView) {
-            //add view to the area
-            area->addView(partView, uiController->activeSublimeWindow()->activeView());
-        }
-        
-        if (!activationParams.testFlag(IDocumentController::DoNotActivate))
-        {
-            uiController->activeSublimeWindow()->activateView(partView);
-        }
-        d->fileOpenRecent->addUrl( url );
-
-        if( applyRange && range.isValid() )
-        {
-            if (range.isEmpty())
-                doc->setCursorPosition( range.start() );
-            else
-                doc->setTextSelection( range );
-        }
-    }
-
-    // Deferred signals, wait until it's all ready first
-    if( emitOpened ) {
-        emit documentOpened( d->documents[url] );
-    }
-
-    if (!activationParams.testFlag(IDocumentController::DoNotActivate))
-        emit documentActivated( doc );
-
-    d->saveAll->setEnabled(true);
-    d->revertAll->setEnabled(true);
-    d->close->setEnabled(true);
-    d->closeAll->setEnabled(true);
-    d->closeAllOthers->setEnabled(true);
-
-    if(doc) {
-        KTextEditor::Cursor activePosition;
-        if(doc->textDocument() && doc->textDocument()->activeView())
-            activePosition = doc->textDocument()->activeView()->cursorPosition();
-        
-        emit documentJumpPerformed(doc, range.start(), previousActiveDocument, previousActivePosition);
-    }
-    
-    return doc;
+    return d->openDocumentInternal( inputUrl, "", range, encoding, activationParams );
 }
 
+
+bool DocumentController::openDocument(IDocument* doc,
+                                      const KTextEditor::Range& range,
+                                      DocumentActivationParams activationParams)
+{
+    return d->openDocumentInternal( doc, range, activationParams );
+}
+
+
 void DocumentController::fileClose()
 {
     IDocument *activeDoc = activeDocument();
@@ -451,17 +579,8 @@ void DocumentController::fileClose()
     {
         UiController *uiController = Core::self()->uiControllerInternal();
         Sublime::View *activeView = uiController->activeSublimeWindow()->activeView();
-        if (activeView->document()->views().count() > 1)
-        {
-            //close only one active view
-            Sublime::View *deletedView = uiController->activeArea()->removeView(activeView);
-            deletedView->deleteLater();
-        }
-        else
-        {
-            //close the document instead
-            activeDoc->close();
-        }
+        
+        uiController->activeArea()->closeView(activeView);
     }
 }
 
@@ -496,8 +615,11 @@ void DocumentController::notifyDocumentClosed(IDocument* doc)
     emit documentClosed(doc);
 }
 
-IDocument * DocumentController::documentForUrl( const KUrl & url ) const
+IDocument * DocumentController::documentForUrl( const KUrl & dirtyUrl ) const
 {
+    //Fix urls that might not be absolute
+    KUrl url(dirtyUrl);
+    url.cleanPath();
     if ( d->documents.contains( url ) )
         return d->documents.value( url );
 
@@ -523,6 +645,7 @@ QList<IDocument*> DocumentController::openDocuments() const
 void DocumentController::activateDocument( IDocument * document, const KTextEditor::Range& range )
 {
     // TODO avoid some code in openDocument?
+    Q_ASSERT(document);
     openDocument(document->url(), range);
 }
 
@@ -540,8 +663,13 @@ bool KDevelop::DocumentController::saveSomeDocuments(const QList< IDocument * >
 {
     if (mode & IDocument::Silent) {
         foreach (IDocument* doc, modifiedDocuments(list)) {
-            bool ret = doc->save(mode);
-            Q_ASSERT(ret);
+            if( !doc->save(mode) )
+            {
+                if( doc )
+                    qWarning() << "!! Could not save document:" << doc->url();
+                else
+                    qWarning() << "!! Could not save document as its NULL";
+            }
             // TODO if (!ret) showErrorDialog() ?
         }
 
@@ -576,19 +704,19 @@ QList< IDocument * > KDevelop::DocumentController::documentsInWindow(MainWindow
     return list;
 }
 
-QList< IDocument * > KDevelop::DocumentController::documentsExclusivelyInWindow(MainWindow * mw) const
+QList< IDocument * > KDevelop::DocumentController::documentsExclusivelyInWindow(MainWindow * mw, bool currentAreaOnly) const
 {
     // Gather a list of all documents which have views only in the given main window
     QList<IDocument*> checkSave;
+    
     foreach (IDocument* doc, openDocuments()) {
         if (Sublime::Document* sdoc = dynamic_cast<Sublime::Document*>(doc)) {
             bool inOtherWindow = false;
 
             foreach (Sublime::View* view, sdoc->views()) {
-                if (view->hasWidget() && view->widget()->window() != mw) {
-                    inOtherWindow = true;
-                    break;
-                }
+                foreach(Sublime::MainWindow* window, Core::self()->uiControllerInternal()->mainWindows())
+                    if(window->containsView(view) && (window != mw || (currentAreaOnly && window == mw && !mw->area()->views().contains(view))))
+                        inOtherWindow = true;
             }
 
             if (!inOtherWindow)
@@ -607,9 +735,9 @@ QList< IDocument * > KDevelop::DocumentController::modifiedDocuments(const QList
     return ret;
 }
 
-bool DocumentController::saveAllDocumentsForWindow(MainWindow* mw, IDocument::DocumentSaveMode mode)
+bool DocumentController::saveAllDocumentsForWindow(KDevelop::MainWindow* mw, KDevelop::IDocument::DocumentSaveMode mode, bool currentAreaOnly)
 {
-    QList<IDocument*> checkSave = documentsExclusivelyInWindow(mw);
+    QList<IDocument*> checkSave = documentsExclusivelyInWindow(mw, currentAreaOnly);
 
     return saveSomeDocuments(checkSave, mode);
 }
@@ -661,29 +789,10 @@ void DocumentController::closeAllOtherDocuments()
             return;
 
         foreach (Sublime::View* view, mw->area()->views()) {
-            if (view != activeView) {
-                if (view->document()->views().count() > 1)
-                {
-                    //close only the view in question
-                    mw->area()->removeView(view);
-                    view->deleteLater();
-                }
-                else
-                {
-                    //close the document instead as no views remain
-                    IDocument* doc = dynamic_cast<IDocument*>(view->document());
-                    if (doc) {
-                        doc->close(IDocument::Discard);
-
-                    } else {
-                        // Fallback, ick
-                        kWarning() << "Tried to close non-IDocument sublime document";
-                        mw->area()->removeView(view);
-                        view->deleteLater();
-                    }
-                }
-            }
+            if (view != activeView)
+                mw->area()->closeView(view);
         }
+        activeView->widget()->setFocus();
     }
 }
 
@@ -706,6 +815,48 @@ QStringList DocumentController::documentTypes() const
     return QStringList() << "Text";
 }
 
+bool DocumentController::isEmptyDocumentUrl(const KUrl &url)
+{
+    QRegExp r(QString("^%1(\\s\\(\\d+\\))?$").arg(EMPTY_DOCUMENT_URL));
+    return r.indexIn(url.prettyUrl()) != -1;
+}
+
+KUrl DocumentController::nextEmptyDocumentUrl()
+{
+    int nextEmptyDocNumber = 0;
+    foreach (IDocument *doc, Core::self()->documentControllerInternal()->openDocuments())
+    {
+        if (DocumentController::isEmptyDocumentUrl(doc->url()))
+        {
+            QRegExp r(QString("^%1\\s\\((\\d+)\\)$").arg(EMPTY_DOCUMENT_URL));
+            if (r.indexIn(doc->url().prettyUrl()) != -1)
+                nextEmptyDocNumber = qMax(nextEmptyDocNumber, r.cap(1).toInt()+1);
+            else
+                nextEmptyDocNumber = qMax(nextEmptyDocNumber, 1);
+        }
+    }
+    
+    KUrl url;
+    if (nextEmptyDocNumber > 0)
+        url = KUrl(QString("%1 (%2)").arg(EMPTY_DOCUMENT_URL).arg(nextEmptyDocNumber));
+    else
+        url = KUrl(EMPTY_DOCUMENT_URL);
+    return url;
+}
+
+IDocumentFactory* DocumentController::factory(const QString& mime) const
+{
+    return d->factories.value(mime);
+}
+
+KTextEditor::Document* DocumentController::globalTextEditorInstance()
+{
+    if(!d->globalTextEditorInstance)
+        d->globalTextEditorInstance = Core::self()->partControllerInternal()->createTextPart();
+    return d->globalTextEditorInstance;
+}
+
+
 }
 
 #include "documentcontroller.moc"
