@@ -38,6 +38,10 @@
 #include "completionsettings.h"
 #include <QThread>
 
+// Maximum length of a string to still consider it as a file extension which we cache
+// This has to be a slow value, so that we don't fill our file extension cache with crap
+static const int maximumCacheExtensionLength = 3;
+
 namespace KDevelop {
 
 
@@ -46,7 +50,7 @@ typedef QHash<QString, QList<ILanguage*> > LanguageCache;
 
 struct LanguageControllerPrivate {
     LanguageControllerPrivate(LanguageController *controller)
-        : dataMutex(QMutex::Recursive), backgroundParser(new BackgroundParser(controller)), m_controller(controller), m_cleanedUp(false) {}
+        : dataMutex(QMutex::Recursive), backgroundParser(new BackgroundParser(controller)), m_cleanedUp(false), m_controller(controller) {}
 
     void documentActivated(KDevelop::IDocument *document)
     {
@@ -76,6 +80,9 @@ struct LanguageControllerPrivate {
     LanguageCache languageCache; //Maps mimetype-names to languages
     typedef QMultiMap<KMimeType::Ptr, ILanguage*> MimeTypeCache;
     MimeTypeCache mimeTypeCache; //Maps mimetypes to languages
+    // fallback cache for file extensions not handled by any pattern
+    typedef QMap<QString, QList<ILanguage*> > FileExtensionCache;
+    FileExtensionCache fileExtensionCache;
 
     BackgroundParser *backgroundParser;
     bool m_cleanedUp;
@@ -99,7 +106,7 @@ ILanguage* LanguageControllerPrivate::addLanguageForSupport(KDevelop::ILanguageS
     QVariant mimetypes = Core::self()->pluginController()->pluginInfo(dynamic_cast<IPlugin*>(languageSupport)).property("X-KDevelop-SupportedMimeTypes");
 
     foreach(const QString& mimeTypeName, mimetypes.toStringList()) {
-        kDebug() << "adding supported mimetype:" << mimeTypeName << "language:" << languageSupport->name();
+        kDebug(9505) << "adding supported mimetype:" << mimeTypeName << "language:" << languageSupport->name();
         languageCache[mimeTypeName] << ret;
         KMimeType::Ptr mime = KMimeType::mimeType(mimeTypeName);
         if(mime) {
@@ -126,6 +133,8 @@ LanguageController::~LanguageController()
 
 void LanguageController::initialize()
 {
+    d->backgroundParser->loadSettings();
+
     // make sure the DUChain is setup before we try to access it from different threads at the same time
     DUChain::self();
 
@@ -187,6 +196,17 @@ ILanguage *LanguageController::language(const QString &name) const
     }
 }
 
+bool isNumeric(const QString& str)
+{
+    int len = str.length();
+    if(len == 0)
+        return false;
+    for(int a = 0; a < len; ++a)
+        if(!str[a].isNumber())
+            return false;
+    return true;
+}
+
 QList<ILanguage*> LanguageController::languagesForUrl(const KUrl &url)
 {
     QMutexLocker lock(&d->dataMutex);
@@ -196,20 +216,26 @@ QList<ILanguage*> LanguageController::languagesForUrl(const KUrl &url)
     if(d->m_cleanedUp)
         return languages;
     
-    QString fileName = url.fileName();
+    const QString fileName = url.fileName();
 
-    ///TODO: evaluate on what the memory impact would be to cache the QRegExp
-    ///      for every pattern
-    ///      right now > 70% in languagesForUrl is spent in setPattern...
+    ///TODO: cache regexp or simple string pattern for endsWith matching
     QRegExp exp("", Qt::CaseInsensitive, QRegExp::Wildcard);
     ///non-crashy part: Use the mime-types of known languages
-    ///Very inefficient right now
     for(LanguageControllerPrivate::MimeTypeCache::const_iterator it = d->mimeTypeCache.constBegin();
         it != d->mimeTypeCache.constEnd(); ++it)
     {
         foreach(QString pattern, it.key()->patterns()) {
-            if(pattern.startsWith('*'))
+            if(pattern.startsWith('*')) {
                 pattern = pattern.mid(1);
+                if (!pattern.contains('*')) {
+                    //optimize: we can skip the expensive QRegExp in this case
+                    //and do a simple string compare (much faster)
+                    if (fileName.endsWith(pattern)) {
+                        languages << *it;
+                    }
+                    continue;
+                }
+            }
 
             exp.setPattern(pattern);
             if(int position = exp.indexIn(fileName)) {
@@ -219,13 +245,42 @@ QList<ILanguage*> LanguageController::languagesForUrl(const KUrl &url)
         }
     }
 
+    if(!languages.isEmpty())
+        return languages;
+
+    // no pattern found, try the file extension cache
+    int extensionStart = fileName.lastIndexOf(QLatin1Char('.'));
+    QString extension;
+    if(extensionStart != -1)
+    {
+        extension = fileName.mid(extensionStart+1);
+        if(extension.size() > maximumCacheExtensionLength || isNumeric(extension))
+            extension = QString();
+    }
+
+    if(!extension.isEmpty())
+    {
+        languages = d->fileExtensionCache.value(extension);
+        if(languages.isEmpty() && d->fileExtensionCache.contains(extension))
+            return languages; // Nothing found, but was in the cache
+    }
+
     //Never use findByUrl from within a background thread, and never load a language support
     //from within the backgruond thread. Both is unsafe, and can lead to crashes
     if(!languages.isEmpty() || QThread::currentThread() != thread())
         return languages;
 
-    ///Crashy and unsafe part: Load missing language-supports
-    KMimeType::Ptr mimeType = KMimeType::findByUrl(url);
+    KMimeType::Ptr mimeType;
+    
+    if(!extension.isEmpty())
+        // If we have recognized a file extension, allow using the file-contents
+        // to look up the type. We will cache it after all.
+        mimeType = KMimeType::findByUrl(url);
+    else
+        // If we have not recognized a file extension, do not allow using the file-contents
+        // to look up the type. We cannot cache the result, and thus we might end up reading
+        // the contents of every single file, which can make the application very unresponsive.
+        mimeType = KMimeType::findByUrl(url, 0, false, true);
 
     LanguageCache::ConstIterator it = d->languageCache.constFind(mimeType->name());
     if (it != d->languageCache.constEnd()) {
@@ -237,17 +292,21 @@ QList<ILanguage*> LanguageController::languagesForUrl(const KUrl &url)
             allPluginsForExtension("ILanguageSupport", constraints);
 
         if (supports.isEmpty()) {
+            kDebug(9505) << "no languages for mimetype:" << mimeType->name();
             d->languageCache.insert(mimeType->name(), QList<ILanguage*>());
         } else {
             foreach (IPlugin *support, supports) {
                 ILanguageSupport* languageSupport = support->extension<ILanguageSupport>();
-                kDebug() << "language-support:" << languageSupport;
+                kDebug(9505) << "language-support:" << languageSupport;
                 if(languageSupport)
                     languages << d->addLanguageForSupport(languageSupport);
             }
         }
     }
 
+    if(!extension.isEmpty())
+        d->fileExtensionCache.insert(extension, languages);
+
     return languages;
 }
 
