@@ -28,6 +28,7 @@ Boston, MA 02110-1301, USA.
 #include <QtGui/QDropEvent>
 #include <QtGui/QMenuBar>
 #include <QtCore/QMimeData>
+#include <QtDBus/QDBusConnection>
 
 #include <KDE/KApplication>
 #include <KDE/KActionCollection>
@@ -37,9 +38,12 @@ Boston, MA 02110-1301, USA.
 #include <ktemporaryfile.h>
 #include <kactioncollection.h>
 #include <kdebug.h>
+#include <ktexteditor/document.h>
 #include <ktexteditor/view.h>
+#include <kshortcutsdialog.h>
 #include <kxmlguifactory.h>
 #include <ktoggleaction.h>
+#include <KWindowSystem>
 
 #include <sublime/area.h>
 #include "shellextension.h"
@@ -73,12 +77,15 @@ void MainWindow::applyMainWindowSettings(const KConfigGroup& config, bool force)
 
 QWidget* MainWindow::customButtonForAreaSwitcher ( Sublime::Area* area )
 {
-    return Core::self()->workingSetControllerInternal()->createSetManagerWidget(this, true, area);
+    return Core::self()->workingSetControllerInternal()->createSetManagerWidget(this, area);
 }
 
 MainWindow::MainWindow( Sublime::Controller *parent, Qt::WFlags flags )
         : Sublime::MainWindow( parent, flags )
 {
+    QDBusConnection::sessionBus().registerObject( "/kdevelop/MainWindow",
+        this, QDBusConnection::ExportScriptableSlots );
+
     setAreaSwitcherCornerWidget(Core::self()->workingSetControllerInternal()->createSetManagerWidget(this));
     setAcceptDrops( true );
     KConfigGroup cg = KGlobal::config()->group( "UiSettings" );
@@ -125,6 +132,18 @@ MainWindow::~ MainWindow()
     Core::self()->uiControllerInternal()->mainWindowDeleted(this);
 }
 
+void MainWindow::ensureVisible()
+{
+    if (isMinimized()) {
+        if (isMaximized()) {
+            showMaximized();
+        } else {
+            showNormal();
+        }
+    }
+    KWindowSystem::forceActiveWindow(winId());
+}
+
 QAction* MainWindow::createCustomElement(QWidget* parent, int index, const QDomElement& element)
 {
     QAction* before = 0L;
@@ -147,20 +166,6 @@ QAction* MainWindow::createCustomElement(QWidget* parent, int index, const QDomE
     return KXMLGUIBuilder::createCustomElement(parent, index, element);
 }
 
-QWidget* MainWindow::createContainer(QWidget* parent, int index,
-    const QDomElement& element, QAction*& containerAction)
-{
-#if KDE_VERSION < KDE_MAKE_VERSION(4, 4, 0)
-    //for KDE < 4.4 we need to remove "Editor" toplevel menu - it will
-    //always be empty because our custom katepartui.rc is not used
-    const QString tagName = element.tagName().toLower();
-    if (tagName == QLatin1String("menu") &&
-            element.attribute(QLatin1String("name")).toUtf8() == "editor")
-        return 0;
-#endif
-    return KXMLGUIBuilder::createContainer(parent, index, element, containerAction);
-}
-
 void MainWindow::dragEnterEvent( QDragEnterEvent* ev )
 {
     if( ev->mimeData()->hasFormat( "text/uri-list" ) && ev->mimeData()->hasUrls() )
@@ -219,7 +224,21 @@ void MainWindow::configureShortcuts()
 {
     ///Workaround for a problem with the actions: Always start the shortcut-configuration in the first mainwindow, then propagate the updated
     ///settings into the other windows
-    Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->configureShortcuts();
+
+
+// We need to bring up the shortcut dialog ourself instead of 
+//      Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->configureShortcuts();
+// so we can connect to the saved() signal to propagate changes in the editor shortcuts
+
+   KShortcutsDialog dlg(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsAllowed, this);
+    foreach (KXMLGUIClient *client, Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->clients())
+    {
+        if(client && !client->xmlFile().isEmpty())
+            dlg.addCollection( client->actionCollection() );
+    }
+    
+    connect(&dlg, SIGNAL(saved()), SLOT(shortcutsChanged()));
+    dlg.configure(true);
 
     QMap<QString, QKeySequence> shortcuts;
     foreach(KXMLGUIClient* client, Core::self()->uiControllerInternal()->mainWindows()[0]->guiFactory()->clients()) {
@@ -240,8 +259,32 @@ void MainWindow::configureShortcuts()
             }
         }
     }
+    
 }
 
+void MainWindow::shortcutsChanged()
+{
+    //propagate shortcut changes to all the opened text documents by reloading the UI XML file
+    IDocument* activeDoc = Core::self()->documentController()->activeDocument();
+    if (!activeDoc || !activeDoc->textDocument()) {
+        return;
+    }
+
+    KTextEditor::View *activeClient = activeDoc->textDocument()->activeView();
+
+    foreach(IDocument * doc, Core::self()->documentController()->openDocuments()) {
+        KTextEditor::Document *textDocument = doc->textDocument();
+        if (textDocument) {
+            foreach(KTextEditor::View *client, textDocument->views()) {
+                if (client != activeClient) {
+                    client->reloadXML();
+                }
+            }
+        }
+    }
+}
+
+
 void MainWindow::initialize()
 {
     KStandardAction::keyBindings(this, SLOT(configureShortcuts()), actionCollection());
@@ -334,14 +377,6 @@ void MainWindow::initializeStatusBar()
     d->setupStatusBar();
 }
 
-
-void MainWindow::setupAreaSelector()
-{
-    Sublime::MainWindow::setupAreaSelector();
-    d->setupAreaSelector();
-}
-
-
 void MainWindow::showErrorMessage(const QString& message, int timeout)
 {
     d->showErrorMessage(message, timeout);
@@ -355,6 +390,7 @@ void MainWindow::tabContextMenuRequested(Sublime::View* view, KMenu* menu)
 
 void MainWindow::tabToolTipRequested(Sublime::View* view, QPoint point)
 {
+    Q_UNUSED(point);
     static QPointer<NavigationToolTip> toolTip;
     
     if(toolTip)
