@@ -38,6 +38,8 @@
 #include <ktexteditor/document.h>
 #include <ktexteditor/modificationinterface.h>
 #include <ktexteditor/codecompletioninterface.h>
+#include <ktexteditor/markinterface.h>
+#include <ktexteditor/configinterface.h>
 
 #include <sublime/area.h>
 #include <sublime/view.h>
@@ -45,7 +47,6 @@
 #include <interfaces/context.h>
 #include <interfaces/contextmenuextension.h>
 
-#include <language/editor/editorintegrator.h>
 #include <language/interfaces/editorcontext.h>
 
 #include "core.h"
@@ -170,7 +171,6 @@ struct TextDocumentPrivate {
             return;
         // Tell the editor integrator first
         m_loaded = true;
-        EditorIntegrator::addDocument( m_textDocument->textDocument() );
         m_textDocument->notifyLoaded();
     }
 
@@ -290,22 +290,28 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
     view = d->document->createView(parent);
 
     if (view) {
-        #if KDE_VERSION < KDE_MAKE_VERSION(4, 4, 60)
-        view->setContextMenu( view->defaultContextMenu() );
-        #endif
         connect(view, SIGNAL(contextMenuAboutToShow(KTextEditor::View*,QMenu*)), this, SLOT(populateContextMenu(KTextEditor::View*,QMenu*)));
 
-        #if KDE_VERSION >= KDE_MAKE_VERSION(4, 4, 0)
         //in KDE >= 4.4 we can use KXMLGuiClient::replaceXMLFile to provide
         //katepart with out own restructured UI configuration
-        const QString katePartUI = KStandardDirs::locate("data", "kdevelop/katepartui.rc");
+        QStringList katePartUIs = KGlobal::mainComponent().dirs()->findAllResources("data", "kdevelop/katepartui.rc");
+        const QString katePartUI = katePartUIs.last();
         const QString katePartLocalUI = KStandardDirs::locateLocal("data", "kdevelop/katepartui.rc");
+        if (!QFile::exists(katePartLocalUI)) {
+            // prevent warning:
+            // kdevelop/kdeui (kdelibs): No such XML file ".../.kde/share/apps/kdevelop/katepartui.rc"
+            QFile::copy(katePartUI, katePartLocalUI);
+        }
         view->replaceXMLFile(katePartUI, katePartLocalUI);
-        #endif
     }
 
     if (KTextEditor::CodeCompletionInterface* cc = dynamic_cast<KTextEditor::CodeCompletionInterface*>(view))
         cc->setAutomaticInvocationEnabled(core()->languageController()->completionSettings()->automaticCompletionEnabled());
+    
+    if (KTextEditor::ConfigInterface *config = qobject_cast<KTextEditor::ConfigInterface*>(view)) {
+        config->setConfigValue("allow-mark-menu", false);
+        config->setConfigValue("default-mark-type", KTextEditor::MarkInterface::BreakpointActive);
+    }
 
     return view;
 }
@@ -371,7 +377,7 @@ bool TextDocument::save(DocumentSaveMode mode)
     }
 
     KUrl urlBeforeSave = d->document->url();
-    if (d->document->save())
+    if (d->document->documentSave())
     {
         if (d->document->url() != urlBeforeSave)
             notifyUrlChanged();
@@ -496,7 +502,6 @@ bool TextDocument::close(DocumentSaveMode mode)
         return false;
 
     if ( d->document ) {
-        KDevelop::EditorIntegrator::removeDocument(d->document);
         delete d->document; //We have to delete the document right now, to prevent random crashes in the event handler
     }
 
@@ -532,8 +537,10 @@ QWidget * KDevelop::TextView::createWidget(QWidget * parent)
     if(d->initialRange.isValid()) {
         if(d->initialRange.isEmpty())
             view->setCursorPosition(d->initialRange.start());
-        else
+        else {
+            view->setCursorPosition(d->initialRange.end());
             view->setSelection(d->initialRange);
+        }
     }
     TextEditorWidget* teWidget = new TextEditorWidget(parent);
     teWidget->setEditorView(view);
@@ -555,9 +562,16 @@ QString KDevelop::TextView::viewState() const
 {
     if( d->editor && d->editor->editorView() )
     {
-        KTextEditor::Cursor cursor = d->editor->editorView()->cursorPosition();
-        return QString("Cursor=%1,%2").arg(cursor.line()).arg(cursor.column());
-    }else
+        if (d->editor->editorView()->selection()) {
+            KTextEditor::Range selection = d->editor->editorView()->selectionRange();
+            return QString("Selection=%1,%2,%3,%4").arg(selection.start().line())
+                                                   .arg(selection.start().column())
+                                                   .arg(selection.end().line())
+                                                   .arg(selection.end().column());
+        } else {
+            KTextEditor::Cursor cursor = d->editor->editorView()->cursorPosition();
+            return QString("Cursor=%1,%2").arg(cursor.line()).arg(cursor.column());
+        }    }else
     {
         kDebug() << "TextView's internal KTE view disappeared!";
         return QString();
@@ -570,12 +584,20 @@ void KDevelop::TextView::setInitialRange(KTextEditor::Range range) {
 
 void KDevelop::TextView::setState(const QString & state)
 {
-    static QRegExp re("Cursor=([\\d]+),([\\d]+)");
-    if (re.exactMatch(state)) {
-        if (d->editor && d->editor->editorView() && re.exactMatch(state))
-            d->editor->editorView()->setCursorPosition(KTextEditor::Cursor(re.cap(1).toInt(), re.cap(2).toInt()));
+    static QRegExp reCursor("Cursor=([\\d]+),([\\d]+)");
+    static QRegExp reSelection("Selection=([\\d]+),([\\d]+),([\\d]+),([\\d]+)");
+    if (reCursor.exactMatch(state)) {
+        if (d->editor && d->editor->editorView())
+            d->editor->editorView()->setCursorPosition(KTextEditor::Cursor(reCursor.cap(1).toInt(), reCursor.cap(2).toInt()));
         else
-            setInitialRange(KTextEditor::Range(KTextEditor::Cursor(re.cap(1).toInt(), re.cap(2).toInt()), 0));
+            setInitialRange(KTextEditor::Range(KTextEditor::Cursor(reCursor.cap(1).toInt(), reCursor.cap(2).toInt()), 0));
+    } else if (reSelection.exactMatch(state)) {
+        KTextEditor::Range range(reSelection.cap(1).toInt(), reSelection.cap(2).toInt(), reSelection.cap(3).toInt(), reSelection.cap(4).toInt());
+        if (d->editor && d->editor->editorView()) {
+            d->editor->editorView()->setCursorPosition(range.end());
+            d->editor->editorView()->setSelection(range);
+        } else
+            setInitialRange(range);
     }
 }
 
